<?php
/************************************************************************
 * MXChange v0.2.1                                    Start: 08/26/2003 *
 * ===============                              Last change: 11/29/2004 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : mysql-manager.php                                *
 * -------------------------------------------------------------------- *
 * Short description : All MySQL-related functions                      *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Alle MySQL-Relevanten Funktionen                 *
 * -------------------------------------------------------------------- *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2008 by Roland Haeder                           *
 * For more information visit: http://www.mxchange.org                  *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	$INC = substr(dirname(__FILE__), 0, strpos(dirname(__FILE__), "/inc") + 4) . "/security.php";
	require($INC);
}

//
function ADD_MODULE_TITLE($mod) {
	global $cacheArray, $_CONFIG;
	$name = ""; $result = false;

	// Is the script installed?
	if (isBooleanConstantAndTrue('mxchange_installed')) {
		// Check if cache is valid
		if ((GET_EXT_VERSION("cache") >= "0.1.2") && (isset($cacheArray['modules']['module'])) && (in_array($mod, $cacheArray['modules']['module']))) {
			// Load from cache
			$name = $cacheArray['modules']['title'][$mod];

			// Update cache hits
			if (getConfig('cache_hits') > 0) { $_CONFIG['cache_hits']++; } else { $_CONFIG['cache_hits'] = 1; }
		} elseif (!EXT_IS_ACTIVE("cache")) {
			// Load from database
			$result = SQL_QUERY_ESC("SELECT title FROM "._MYSQL_PREFIX."_mod_reg WHERE module='%s' LIMIT 1", array($mod), __FILE__, __LINE__);
			list($name) = SQL_FETCHROW($result);
			SQL_FREERESULT($result);
		}
	} // END - if

	// Trim name
	$name = trim($name);

	// Still no luck or empty title?
	if (empty($name)) {
		// No name found
		$name = sprintf("%s (%s)", LANG_UNKNOWN_MODULE, $mod);
		if (SQL_NUMROWS($result) == 0) {
			// Add module to database
			$dummy = CHECK_MODULE($mod);
		} // END - if
	} // END - if

	// Return name
	return $name;
}

// Check validity of a given module name (no file extension)
function CHECK_MODULE ($mod) {
	// We need them now here...
	global $cacheArray, $_CONFIG, $cacheInstance;

	// Filter module name (names with low chars and underlines are fine!)
	$mod = preg_replace("/[^a-z_]/", "", $mod);

	// Check for prefix is a extension...
	$modSplit = explode("_", $mod);
	$extension = ""; $mod_chk = $mod;
	//* DEBUG: */ echo __LINE__."*".count($modSplit)."*/".$mod."*<br />";
	if (count($modSplit) == 2) {
		// Okay, there is a seperator (_) in the name so is the first part a module?
		//* DEBUG: */ echo __LINE__."*".$modSplit[0]."*<br />";
		if (EXT_IS_ACTIVE($modSplit[0])) {
			// The prefix is an extension's name, so let's set it
			$extension = $modSplit[0]; $mod = $modSplit[1];
		}
	}

	// Major error in module registry is the default
	$ret = "major";

	// Check if script is installed if not return a "done" to prevent some errors
	if ((!isBooleanConstantAndTrue('mxchange_installed')) || (isBooleanConstantAndTrue('mxchange_installing')) || (!isBooleanConstantAndTrue('admin_registered'))) return "done";

	// Check if cache is latest version
	$locked = "Y"; $hidden = "N"; $admin = "N"; $mem = "N"; $found = false;
	if ((GET_EXT_VERSION("cache") >= "0.1.2") && (isset($cacheArray['modules']['module'])) && (is_array($cacheArray['modules']['module']))) {
		// Is the module cached?
		if (isset($cacheArray['modules']['locked'][$mod_chk])) {
			// Check cache
			$locked = $cacheArray['modules']['locked'][$mod_chk];
			$hidden = $cacheArray['modules']['hidden'][$mod_chk];
			$admin  = $cacheArray['modules']['admin_only'][$mod_chk];
			$mem    = $cacheArray['modules']['mem_only'][$mod_chk];

			// Update cache hits
			if (getConfig('cache_hits') > 0) { $_CONFIG['cache_hits']++; } else { $_CONFIG['cache_hits'] = 1; }
			$found = true;
		} else {
			// No, then we have to update it!
			$ret = "cache_miss";
		}
	} elseif (!EXT_IS_ACTIVE("cache")) {
		// Check for module in database
		$result = SQL_QUERY_ESC("SELECT locked, hidden, admin_only, mem_only FROM "._MYSQL_PREFIX."_mod_reg WHERE module='%s' LIMIT 1", array($mod_chk), __FILE__, __LINE__);
		if (SQL_NUMROWS($result) == 1) {
			// Read data
			list($locked, $hidden, $admin, $mem) = SQL_FETCHROW($result);
			SQL_FREERESULT($result);
			$found = true;
		}
	}

	// Check returned values against current access permissions
	//
	//  Admin access            ----- Guest access -----           --- Guest   or   member? ---
	if ((IS_ADMIN()) || (($locked == "N") && ($admin == "N") && (($mem == "N") || (IS_MEMBER())))) {
		// If you are admin you are welcome for everything!
		$ret = "done";
	} elseif ($locked == "Y") {
		// Module is locked
		$ret = "locked";
	} elseif (($mem == "Y") && (!IS_MEMBER())) {
		// You have to login first!
		$ret = "mem_only";
	} elseif (($admin == "Y") && (!IS_ADMIN())) {
		// Only the Admin is allowed to enter this module!
		$ret = "admin_only";
	}

	// Still no luck or not found?
	if (($ret == "major") || ($ret == "cache_miss") || (!$found)) {
		//              ----- Legacy module -----                                   ---- Module in base folder  ----                       --- Module with extension's name ---
		if ((FILE_READABLE(sprintf("%sinc/modules/%s.php", PATH, $mod))) || (FILE_READABLE(sprintf("%s%s.php", PATH, $mod))) || (FILE_READABLE(sprintf("%s%s/%s.php", PATH, $extension, $mod)))) {
			// Data is missing so we add it
			if (GET_EXT_VERSION("sql_patches") >= "0.3.6") {
				// Since 0.3.6 we have a has_menu column, this took me a half hour
				// to find a loop here... *sigh*
				$result = SQL_QUERY_ESC("INSERT INTO "._MYSQL_PREFIX."_mod_reg
(module, locked, hidden, mem_only, admin_only, has_menu) VALUES
('%s','Y','N','N','N','N')", array($mod_chk), __FILE__, __LINE__);
			} else {
				// Wrong/missing sql_patches!
				$result = SQL_QUERY_ESC("INSERT INTO "._MYSQL_PREFIX."_mod_reg
(module, locked, hidden, mem_only, admin_only) VALUES
('%s','Y','N','N','N')", array($mod_chk), __FILE__, __LINE__);
			}

			// Everthing is fine?
			if (SQL_AFFECTEDROWS() < 1) {
				// Something bad happend!
				return "major";
			} // END - if

			// Destroy cache here
			REBUILD_CACHE("mod_reg", "modreg");

			// And reload data
			$ret = CHECK_MODULE($mod_chk);
		} else {
			// Module not found we don't add it to the database
			$ret = "404";
		}
	} // END - if

	// Return the value
	return $ret;
}

// Add menu description pending on given file name (without path!)
function ADD_DESCR($ACC_LVL, $file, $return = false, $output = true) {
	global $NAV_DEPTH, $_CONFIG;
	// Use only filename of the file ;)
	$file = basename($file);

	// Init variables
	$LINK_ADD = ""; $OUT = ""; $AND = "";

	// First we have to do some analysis...
	if (substr($file, 0, 7) == "action-") {
		// This is an action file!
		$type = "action";
		$search = substr($file, 7);
		switch ($ACC_LVL)
		{
		case "admin":
			$modCheck = "admin";
			break;

		case "sponsor":
		case "guest":
		case "member":
			$modCheck = $GLOBALS['module'];
			break;
		}
		$AND = " AND (what='' OR what IS NULL)";
	} elseif (substr($file, 0, 5) == "what-") {
		// This is an admin what file!
		$type = "what";
		$search = substr($file, 5);
		$AND = "";
		switch ($ACC_LVL)
		{
		case "admin":
			$modCheck = "admin";
			break;

		case "guest":
		case "member":
			$modCheck = $GLOBALS['module'];
			if (!IS_ADMIN()) {
				$AND = " AND visible='Y' AND locked='N'";
			}
			break;
		}
		$dummy = substr($search, 0, -4);
		$AND .= " AND action='".GET_ACTION($ACC_LVL, $dummy)."'";
	} elseif (($ACC_LVL == "sponsor") || ($ACC_LVL == "engine")) {
		// Sponsor / engine menu
		$type = "what";
		$search = $file;
		$modCheck = $GLOBALS['module'];
		$AND = "";
	} else {
		// Other
		$type = "menu";
		$search = $file;
		$modCheck = $GLOBALS['module'];
		$AND = "";
	}
	if ((!isset($NAV_DEPTH)) && (!$return)) {
		$NAV_DEPTH = 0;
		$prefix = "<DIV class=\"you_are_here\">".YOU_ARE_HERE."&nbsp;<STRONG><A class=\"you_are_here\" href=\"".URL."/modules.php?module=".$GLOBALS['module'].$LINK_ADD."\">Home</A></STRONG>";
	} else {
		if (!$return) $NAV_DEPTH++;
		$prefix = "";
	}

	$prefix .= "&nbsp;-&gt;&nbsp;";

	// We need to remove .php and the end
	if (substr($search, -4, 4) == ".php") {
		// Remove the .php
		$search = substr($search, 0, -4);
	} // END - i

	// Get the title from menu
	$result = SQL_QUERY_ESC("SELECT title FROM "._MYSQL_PREFIX."_%s_menu WHERE %s='%s' ".$AND." LIMIT 1",
		array($ACC_LVL, $type, $search), __FILE__, __LINE__);

	// Menu found?
	if (SQL_NUMROWS($result) == 1) {
		// Load title
		list($ret) = SQL_FETCHROW($result);

		// Shall we return it?
		if ($return) {
			// Return title
			return $ret;
		} elseif (((GET_EXT_VERSION("sql_patches") >= "0.2.3") && (getConfig('youre_here') == "Y")) || ((IS_ADMIN()) && ($modCheck == "admin"))) {
			// Output HTML code
			$OUT = $prefix."<STRONG><A class=\"you_are_here\" href=\"".URL."/modules.php?module=".$modCheck."&amp;".$type."=".$search.$LINK_ADD."\">".$ret."</A></STRONG>\n";

			// Can we close the you-are-here navigation?
			//* DEBUG: */ echo __LINE__."*".$type."/".$GLOBALS['what']."*<br />\n";
			//* DEBUG: */ die("<pre>".print_r($_CONFIG, true)."</pre>");
			if (($type == "what") || (($type == "action") && ((!isset($GLOBALS['what'])) || ($GLOBALS['what'] == "overview")))) {
				//* DEBUG: */ echo __LINE__."+".$type."+<br />\n";
				// Add closing div and br-tag
				$OUT .= "</div><br />\n";
				$NAV_DEPTH = "0";

				// Run the filter chain
				$ret = RUN_FILTER('post_youhere_line', array('access_level' => $ACC_LVL, 'type' => $type, 'content' => ""));
				$OUT .= $ret['content'];
			} // END - if
		}
	} // END - if

	// Free result
	SQL_FREERESULT($result);

	// Return or output HTML code?
	if ($output) {
		// Output HTML code here
		OUTPUT_HTML($OUT);
	} else {
		// Return HTML code
		return $OUT;
	}
}
//
function ADD_MENU($MODE, $act, $wht) {
	global $_CONFIG;

	// Init some variables
	$main_cnt = 0;
	$AND = "";
	$main_action = "";
	$sub_what = "";

	if (!VALIDATE_MENU_ACTION($MODE, $act, $wht, true)) return CODE_MENU_NOT_VALID;

	// Non-admin shall not see all menus
	if (!IS_ADMIN()) {
		$AND = " AND visible='Y' AND locked='N'";
	}

	// Load SQL data and add the menu to the output stream...
	$result_main = SQL_QUERY_ESC("SELECT title, action FROM "._MYSQL_PREFIX."_%s_menu WHERE (what='' OR what IS NULL)".$AND." ORDER BY sort",
	 array($MODE), __FILE__, __LINE__);
	//* DEBUG: */ echo __LINE__."/".$main_cnt."/".$main_action."/".$sub_what.":".$GLOBALS['what']."*<br />\n";
	if (SQL_NUMROWS($result_main) > 0) {
		OUTPUT_HTML("<TABLE border=\"0\" cellspacing=\"0\" cellpadding=\"0\" class=\"".$MODE."_menu\">");
		// There are menus available, so we simply display them... :)
		while (list($main_title, $main_action) = SQL_FETCHROW($result_main)) {
			//* DEBUG: */ echo __LINE__."/".$main_cnt."/".$main_action."/".$sub_what.":".$GLOBALS['what']."*<br />\n";
			// Init variables
			$BLOCK_MODE = false; $act = $main_action;

			// Prepare content
			$content = array(
				'action' => $main_action,
				'title'  => $main_title
			);

			// Load menu header template
			LOAD_TEMPLATE($MODE."_menu_title", false, $content);

			$result_sub = SQL_QUERY_ESC("SELECT title, what FROM "._MYSQL_PREFIX."_%s_menu WHERE action='%s' AND what != '' AND what IS NOT NULL ".$AND." ORDER BY sort",
			 array($MODE, $main_action), __FILE__, __LINE__);
			$ctl = SQL_NUMROWS($result_sub);
			if ($ctl > 0) {
				$cnt=0;
				while (list($sub_title, $sub_what) = SQL_FETCHROW($result_sub)) {
					// Init content
					$content = "";

					// Full file name for checking menu
					//* DEBUG: */ echo __LINE__.":!!!!".$sub_what."!!!<br />\n";
					$test_inc = sprintf("%sinc/modules/%s/what-%s.php", PATH, $MODE, $sub_what);
					$test = (FILE_READABLE($test_inc));
					if ($test) {
						if ((!empty($wht)) && (($wht == $sub_what))) {
							$content = "<STRONG>";
						}

						// Navigation link
						$content .= "<A name=\"menu\" class=\"menu_blur\" href=\"".URL."/modules.php?module=".$GLOBALS['module']."&amp;what=".$sub_what.ADD_URL_DATA("")."\" target=\"_self\">";
					} else {
						$content .= "<I>";
					}

					// Menu title
					$content .= getConfig('menu_blur_spacer') . $sub_title;

					if ($test) {
						$content .= "</A>";
					} else {
						$content .= "</I>";
					}

					if ((!empty($wht)) && (($wht == $sub_what))) {
						$content .= "</STRONG>";
					}
					$wht = $sub_what; $cnt++;
					// Prepare array
					$content =  array(
						'menu' => $content,
						'what' => $sub_what
					);

					// Add regular menu row or bottom row?
					if ($cnt < $ctl) {
						LOAD_TEMPLATE($MODE."_menu_row", false, $content);
					} else {
						LOAD_TEMPLATE($MODE."_menu_bottom", false, $content);
					}
				}
			} else {
				// This is a menu block... ;-)
				$BLOCK_MODE = true;
				$INC_BLOCK = sprintf("%sinc/modules/%s/action-%s.php", PATH, $MODE, $main_action);
				if (FILE_READABLE($INC_BLOCK)) {
					// Load include file
					if ((!EXT_IS_ACTIVE($main_action)) || ($main_action == "online")) OUTPUT_HTML("<TR>
  <TD class=\"".$MODE."_menu_whats\">");
					//* DEBUG: */ echo __LINE__."/".$main_cnt."/".$main_action."/".$sub_what.":".$GLOBALS['what']."*<br />\n";
					include ($INC_BLOCK);
					//* DEBUG: */ echo __LINE__."/".$main_cnt."/".$main_action."/".$sub_what.":".$GLOBALS['what']."*<br />\n";
					if ((!EXT_IS_ACTIVE($main_action)) || ($main_action == "online")) OUTPUT_HTML("  </TD>
</TR>");
				}
				//* DEBUG: */ echo __LINE__."/".$main_cnt."/".$main_action."/".$sub_what.":".$GLOBALS['what']."*<br />\n";
			}
			$main_cnt++;
			//* DEBUG: */ echo __LINE__."/".$main_cnt."/".$main_action."/".$sub_what.":".$GLOBALS['what']."*<br />\n";
			if (SQL_NUMROWS($result_main) > $main_cnt) 	OUTPUT_HTML("<TR><TD class=\"".$MODE."_menu_seperator\"></TD></TR>");
		}

		// Free memory
		SQL_FREERESULT($result_main);

		// Close table
		//* DEBUG: */ echo __LINE__."/".$main_cnt."/".$main_action."/".$sub_what.":".$GLOBALS['what']."*<br />\n";
		OUTPUT_HTML("</TABLE>");
	}
}
// This patched function will reduce many SELECT queries for the specified or current admin login
function IS_ADMIN($admin="")
{
	global $cacheArray, $_CONFIG;
	$ret = false; $passCookie = ""; $valPass = "";
	//* DEBUG: */ echo __LINE__."ADMIN:".$admin."<br />";

	// If admin login is not given take current from cookies...
	if ((empty($admin)) && (isSessionVariableSet('admin_login')) && (isSessionVariableSet('admin_md5'))) {
		// Get admin login and password from session/cookies
		$admin = get_session('admin_login');
		$passCookie = get_session('admin_md5');
	}
	//* DEBUG: */ echo __LINE__."ADMIN:".$admin."/".$passCookie."<br />";

	// Search in array for entry
	if (isset($cacheArray['admin_hash'])) {
		// Use cached string
		$valPass = $cacheArray['admin_hash'];
	} elseif ((!empty($passCookie)) && (isset($cacheArray['admins']['password'][$admin])) && (!empty($admin))) {
		// Count cache hits
		if (getConfig('cache_hits') > 0) { $_CONFIG['cache_hits']++; } else { $_CONFIG['cache_hits'] = 1; }

		// Login data is valid or not?
		$valPass = generatePassString($cacheArray['admins']['password'][$admin]);

		// Cache it away
		$cacheArray['admin_hash'] = $valPass;
	} elseif ((!empty($admin)) && ((!EXT_IS_ACTIVE("cache"))) || (!isset($cacheArray['admins']['password'][$admin]))) {
		// Search for admin
		$result = SQL_QUERY_ESC("SELECT HIGH_PRIORITY password FROM "._MYSQL_PREFIX."_admins WHERE login='%s' LIMIT 1",
			array($admin), __FILE__, __LINE__);

		// Is he admin?
		$passDB = "";
		if (SQL_NUMROWS($result) == 1) {
			// Admin login was found so let's load password from DB
			list($passDB) = SQL_FETCHROW($result);

			// Temporary cache it
			$cacheArray['admins']['password'][$admin] = $passDB;

			// Generate password hash
			$valPass = generatePassString($passDB);
		} // END - if

		// Free memory
		SQL_FREERESULT($result);
	}

	if (!empty($valPass)) {
		// Check if password is valid
		//* DEBUG: */ print __FUNCTION__."*".$valPass."/".$passCookie."*<br />\n";
		$ret = (($valPass == $passCookie) || ((strlen($valPass) == 32) && ($valPass == md5($passCookie))) || (($valPass == "*FAILED*") && (!EXT_IS_ACTIVE("cache"))));
	}

	// Return result of comparision
	//* DEBUG: */ if (!$ret) echo __LINE__."OK!<br />";
	return $ret;
}
//
function ADD_MAX_RECEIVE_LIST($MODE, $default="", $return=false)
{
	global $_POST;
	$OUT = "";
	switch ($MODE)
	{
	case "guest":
		// Guests (in the registration form) are not allowed to select 0 mails per day.
		$result = SQL_QUERY("SELECT value, comment FROM "._MYSQL_PREFIX."_max_receive WHERE value > 0 ORDER BY value", __FILE__, __LINE__);
		if (SQL_NUMROWS($result) > 0)
		{
			$OUT = "";
  			while (list($value, $comment) = SQL_FETCHROW($result))
			{
				$OUT .= "      <OPTION value=\"".$value."\"";
				if ($_POST['max_mails'] == $value) $OUT .= " selected=\"selected\"";
				$OUT .= ">".$value." ".PER_DAY;
				if (!empty($comment)) $OUT .= " (".$comment.")";
				$OUT .= "</OPTION>\n";
			}
			define('__MAX_RECEIVE_OPTIONS', $OUT);

			// Free memory
			SQL_FREERESULT($result);
			$OUT = LOAD_TEMPLATE("guest_receive_table", true);
		}
		 else
		{
			// Maybe the admin has to setup some maximum values?
		}
		break;

	case "member":
		// Members are allowed to set to zero mails per day (we will change this soon!)
		$result = SQL_QUERY("SELECT value, comment FROM "._MYSQL_PREFIX."_max_receive ORDER BY value", __FILE__, __LINE__);
		if (SQL_NUMROWS($result) > 0)
		{
			$OUT = "";
  			while (list($value, $comment) = SQL_FETCHROW($result))
			{
				$OUT .= "      <OPTION value=\"".$value."\"";
				if ($default == $value) $OUT .= " selected=\"selected\"";
				$OUT .= ">".$value." ".PER_DAY;
				if (!empty($comment)) $OUT .= " (".$comment.")";
				$OUT .= "</OPTION>\n";
			}
			define('__MAX_RECEIVE_OPTIONS', $OUT);
			SQL_FREERESULT($result);
			$OUT = LOAD_TEMPLATE("member_receive_table", true);
		}
		 else
		{
			// Maybe the admin has to setup some maximum values?
			$OUT = LOAD_TEMPLATE("admin_settings_saved", true, NO_MAX_VALUES);
		}
		break;
	}
	if ($return)
	{
		// Return generated HTML code
		return $OUT;
	}
	 else
	{
		// Output directly (default)
		OUTPUT_HTML($OUT);
	}
}
//
function SEARCH_EMAIL_USERTAB($email)
{
	$ret = false;
	$result = SQL_QUERY_ESC("SELECT userid FROM `"._MYSQL_PREFIX."_user_data` WHERE email LIKE '{PER}%s{PER}' LIMIT 1", array($email), __FILE__, __LINE__);
	if (SQL_NUMROWS($result) == 1) $ret = true;
	SQL_FREERESULT($result);
	return $ret;
}
//
function WHAT_IS_VALID($act, $wht, $type="guest")
{
	if (IS_ADMIN())
	{
		// Everything is valid to the admin :-)
		return true;
	}
	 else
	{
		$ret = false;
		$result = SQL_QUERY_ESC("SELECT id FROM "._MYSQL_PREFIX."_%s_menu WHERE action='%s' AND what='%s' AND locked='N' LIMIT 1", array($type, $act, $wht), __FILE__, __LINE__);
		// Is "what" valid?
		if (SQL_NUMROWS($result) == 1) $ret = true;
		SQL_FREERESULT($result);
		return $ret;
	}
}
//
function IS_MEMBER()
{
	global $status, $LAST, $cacheArray;
	if (!is_array($LAST)) $LAST = array();
	$ret = false;

	// is the cache entry there?
	if (isset($cacheArray['is_member'])) {
		// Then return it
		return $cacheArray['is_member'];
	} // END - if

	// Fix "deleted" cookies first
	FIX_DELETED_COOKIES(array('userid','u_hash','lifetime'));

	// Are cookies set?
	if ((!empty($GLOBALS['userid'])) && (isSessionVariableSet('u_hash')) && (isSessionVariableSet('lifetime')) && (defined('COOKIE_PATH')))
	{
		// Cookies are set with values, but are they valid?
		$result = SQL_QUERY_ESC("SELECT password, status, last_module, last_online FROM `"._MYSQL_PREFIX."_user_data` WHERE userid=%s LIMIT 1",
		 array($GLOBALS['userid']), __FILE__, __LINE__);
		if (SQL_NUMROWS($result) == 1)
		{
			// Load data from cookies
			list($password, $status, $mod, $onl) = SQL_FETCHROW($result);

			// Validate password by created the difference of it and the secret key
			$valPass = generatePassString($password);

			// Transfer last module and online time
			if ((!empty($mod)) && (empty($LAST['module']))) { $LAST['module'] = $mod; $LAST['online'] = $onl; }

			// So did we now have valid data and an unlocked user?
			//* DEBUG: */ echo $valPass."<br />".get_session('u_hash')."<br />";
			if (($status == "CONFIRMED") && ($valPass == get_session('u_hash'))) {
				// Account is confirmed and all cookie data is valid so he is definely logged in! :-)
				$ret = true;
			} else {
				// Maybe got locked etc.
				//* DEBUG: */ echo __LINE__."!!!<br />";
				destroy_user_session();

				// Reset userid
				$GLOBALS['userid'] = 0;
			}
		} else {
			// Cookie data is invalid!
			//* DEBUG: */ echo __LINE__."***<br />";
			destroy_user_session();

			// Reset userid
			$GLOBALS['userid'] = 0;
		}

		// Free memory
		SQL_FREERESULT($result);
	} else {
		// Cookie data is invalid!
		//* DEBUG: */ echo __LINE__."///<br />";
		destroy_user_session();

		// Reset userid
		$GLOBALS['userid'] = 0;
	}

	// Cache status
	$cacheArray['is_member'] = $ret;

	// Return status
	return $ret;
}
//
function VALIDATE_MENU_ACTION ($MODE, $act, $wht, $UPDATE=false)
{
	$ret = false;
	$ADD = "";
	if ((!IS_ADMIN()) && ($MODE != "admin")) $ADD = " AND locked='N'";
	//* DEBUG: */ echo __LINE__.":".$MODE."/".$act."/".$wht."*<br />\n";
	if (($MODE != "admin") && ($UPDATE))
	{
		// Update guest or member menu
		$SQL = SQL_QUERY_ESC("UPDATE "._MYSQL_PREFIX."_%s_menu SET counter=counter+1 WHERE action='%s' AND what='%s'".$ADD." LIMIT 1",
		 array($MODE, $act, $wht), __FILE__, __LINE__, false);
	}
	 elseif ($wht != "overview")
	{
		// Other actions
		$SQL = SQL_QUERY_ESC("SELECT id, what FROM "._MYSQL_PREFIX."_%s_menu WHERE action='%s'".$ADD." ORDER BY action DESC LIMIT 1",
		 array($MODE, $act), __FILE__, __LINE__, false);
	}
	 else
	{
		// Admin login overview
		$SQL = SQL_QUERY_ESC("SELECT id, what FROM "._MYSQL_PREFIX."_%s_menu WHERE action='%s' AND (what='' OR what IS NULL)".$ADD." ORDER BY action DESC LIMIT 1",
		 array($MODE, $act), __FILE__, __LINE__, false);
	}

	// Run SQL command
	$result = SQL_QUERY($SQL, __FILE__, __LINE__);
	if ($UPDATE)
	{
		if (SQL_AFFECTEDROWS() == 1) $ret = true;
		//* DEBUG: */ debug_print_backtrace();
	}
	 else
	{
		if (SQL_NUMROWS($result) == 1) {
			list($id, $wht2) = SQL_FETCHROW($result);
			//* DEBUG: */ echo __LINE__."+".$SQL."+<br />\n";
			//* DEBUG: */ echo __LINE__."*".$id."/".$wht."/".$wht2."*<br />\n";
			$ret = true;
		}
	}

	// Free memory
	SQL_FREERESULT($result);

	// Return result
	return $ret;
}
//
function GET_MOD_DESCR($MODE, $wht, $column="what")
{
	// Fix empty "what"
	if (empty($wht)) {
		$wht = "welcome";
		if (getConfig('index_home') != "") $wht = getConfig('index_home');
	} // END - if

	// Default is not found
	$ret = "??? (".$wht.")";

	// Look for title
	$result = SQL_QUERY_ESC("SELECT title FROM "._MYSQL_PREFIX."_%s_menu WHERE %s='%s' LIMIT 1",
		array($MODE, $column, $wht), __FILE__, __LINE__);

	// Is there an entry?
	if (SQL_NUMROWS($result) == 1) {
		// Fetch the title
		list($ret) = SQL_FETCHROW($result);
	} // END - if

	// Free result
	SQL_FREERESULT($result);
	return $ret;
}
//
function SEND_MODE_MAILS($mod, $modes)
{
	global $_CONFIG, $DATA;

	// Load hash
	$result_main = SQL_QUERY_ESC("SELECT password FROM `"._MYSQL_PREFIX."_user_data` WHERE userid=%s AND status='CONFIRMED' LIMIT 1",
		array($GLOBALS['userid']), __FILE__, __LINE__);
	if (SQL_NUMROWS($result_main) == 1) {
		// Load hash from database
		list($hashDB) = SQL_FETCHROW($result_main);

		// Extract salt from cookie
		$salt = substr(get_session('u_hash'), 0, -40);

		// Now let's compare passwords
		$hash = generatePassString($hashDB);
		if (($hash == get_session('u_hash')) || ($_POST['pass1'] == $_POST['pass2'])) {
			// Load user's data
			$result = SQL_QUERY_ESC("SELECT gender, surname, family, street_nr, country, zip, city, email FROM `"._MYSQL_PREFIX."_user_data` WHERE userid=%s AND password='%s' LIMIT 1",
				array($GLOBALS['userid'], $hashDB), __FILE__, __LINE__);
			if (SQL_NUMROWS($result) == 1) {
				// Load the data
				$DATA = SQL_FETCHROW($result);

				// Free result
				SQL_FREERESULT($result);

				// Translate gender
				$DATA[0] = TRANSLATE_GENDER($DATA[0]);

				// Clear/init the content variable
				$content = "";
				$DATA['info'] = "";

				switch ($mod)
				{
				case "mydata":
					foreach ($modes as $mode) {
						switch ($mode)
						{
						case "normal": break; // Do not add any special lines

						case "email": // Email was changed!
							$content = MEMBER_CHANGED_EMAIL.": ".$_POST['old_addy']."\n";
							break;

						case "pass": // Password was changed
							$content = MEMBER_CHANGED_PASS."\n";
							break;

						default:
							DEBUG_LOG(__FUNCTION__, __LINE__, sprintf("Unknown mode %s detected.", $mode));
							$content = MEMBER_UNKNOWN_MODE.": ".$mode."\n\n";
							break;
						} // END - switch
					} // END - if

					if (EXT_IS_ACTIVE("country")) {
						// Replace code with description
						$DATA[4] = COUNTRY_GENERATE_INFO($_POST['country_code']);
					} // END - if

					// Load template
					$msg = LOAD_EMAIL_TEMPLATE("member_mydata_notify", $content, $GLOBALS['userid']);

					if (getConfig('admin_notify') == "Y") {
						// The admin needs to be notified about a profile change
						$msg_admin = "admin_mydata_notify";
						$sub_adm   = ADMIN_CHANGED_DATA;
					} else {
						// No mail to admin
						$msg_admin = "";
						$sub_adm   = "";
					}

					// Set subject lines
					$sub_mem = MEMBER_CHANGED_DATA;

					// Output success message
					$content = "<SPAN class=\"member_done\">".MYDATA_MAIL_SENT."</SPAN>";
					break;

				default:
					DEBUG_LOG(__FUNCTION__, __LINE__, sprintf("Unsupported module %s detected.", $mod));
					$content = "<SPAN class=\"member_failed\">".UNKNOWN_MODULE."</SPAN>";
					break;
				} // END - switch
			} else {
				// Could not load profile data
				$content = "<SPAN class=\"member_failed\">".MEMBER_CANNOT_LOAD_PROFILE."</SPAN>";
			}
		} else {
			// Passwords mismatch
			$content = "<SPAN class=\"member_failed\">".MEMBER_PASSWORD_ERROR."</SPAN>";
		}
	} else {
		// Could not load profile
		$content = "<SPAN class=\"member_failed\">".MEMBER_CANNOT_LOAD_PROFILE."</SPAN>";
	}

	// Send email to user if required
	if ((!empty($sub_mem)) && (!empty($msg))) {
		// Send member mail
		SEND_EMAIL($DATA[7], $sub_mem, $msg);
	} // END - if

	// Send only if no other error has occured
	if (empty($content)) {
		if ((!empty($sub_adm)) && (!empty($msg_admin))) {
			// Send admin mail
			SEND_ADMIN_NOTIFICATION($sub_adm, $msg_admin, $content, $GLOBALS['userid']);
		} elseif (getConfig('admin_notify') == "Y") {
			// Cannot send mails to admin!
			$content = CANNOT_SEND_ADMIN_MAILS;
		} else {
			// No mail to admin
			$content = "<SPAN class=\"member_done\">".MYDATA_MAIL_SENT."</SPAN>";
		}
	} // END - if

	// Load template
	LOAD_TEMPLATE("admin_settings_saved", false, $content);
}
// Update module counter
function COUNT_MODULE($mod) {
	if ($mod != "css") {
		// Do count all other modules but not accesses on CSS file css.php!
		$result = SQL_QUERY_ESC("UPDATE "._MYSQL_PREFIX."_mod_reg SET clicks=clicks+1 WHERE module='%s' LIMIT 1",
			array($mod), __FILE__, __LINE__);
	} // END - if
}
// Get action value from mode (admin/guest/member) and what-value
function GET_ACTION ($MODE, &$wht)
{
	global $ret, $_CONFIG;
	// @DEPRECATED Init status
	$ret = "";

	//* DEBUG: */ echo __LINE__."=".$MODE."/".$wht."/".$GLOBALS['action']."=<br />";
	if ((empty($wht)) && ($MODE != "admin")) {
		$wht = "welcome";
		if (getConfig('index_home') != "") $wht = getConfig('index_home');
	} // END - if

	if ($MODE == "admin") {
		// Action value for admin area
		if (!empty($GLOBALS['action'])) {
			// Get it directly from URL
			return $GLOBALS['action'];
		} elseif (($wht == "overview") || (empty($GLOBALS['what']))) {
			// Default value for admin area
			$ret = "login";
		}
	} elseif (!empty($GLOBALS['action'])) {
		// Get it directly from URL
		return $GLOBALS['action'];
	}
	//* DEBUG: */ echo __FUNCTION__."(<font color=\"#0000aa\">".__LINE__."</font>): ret=".$ret."<br />\n";

	if (MODULE_HAS_MENU($MODE)) {
		// Rewriting modules to menu
		switch ($MODE) {
			case "index": $MODE = "guest";  break;
			case "login": $MODE = "member"; break;
		} // END - switch

		// Guest and member menu is "main" as the default
		if (empty($ret)) $ret = "main";

		// Load from database
		$result = SQL_QUERY_ESC("SELECT action FROM "._MYSQL_PREFIX."_%s_menu WHERE what='%s' LIMIT 1",
			array($MODE, $wht), __FILE__, __LINE__);
		if (SQL_NUMROWS($result) == 1) {
			// Load action value and pray that this one is the right you want... ;-)
			list($ret) = SQL_FETCHROW($result);
		} // END - if

		// Free memory
		SQL_FREERESULT($result);
	} // END - if

	// Return action value
	return $ret;
}

// Get category name back
function GET_CATEGORY ($cid) {
	// Default is not found
	$ret = _CATEGORY_404;

	// Is the category id set?
	if ($cid == "0") {
		// No category
		$ret = _CATEGORY_NONE;
	} elseif ($cid > 0) {
		// Lookup the category in database
		$result = SQL_QUERY_ESC("SELECT cat FROM "._MYSQL_PREFIX."_cats WHERE id=%s LIMIT 1",
			array(bigintval($cid)), __FILE__, __LINE__);
		if (SQL_NUMROWS($result) == 1) {
			// Category found... :-)
			list($ret) = SQL_FETCHROW($result);
		} // END - if

		// Free result
		SQL_FREERESULT($result);
	} // END - if

	// Return result
	return $ret;
}

// Get a string of "mail title" and price back
function GET_PAYMENT ($pid, $full=false) {
	// Default is not found
	$ret = _PAYMENT_404;

	// Load payment data
	$result = SQL_QUERY_ESC("SELECT mail_title, price FROM "._MYSQL_PREFIX."_payments WHERE id=%s LIMIT 1",
		array(bigintval($pid)), __FILE__, __LINE__);
	if (SQL_NUMROWS($result) == 1) {
		// Payment type found... :-)
		if (!$full) {
			// Return only title
			list($ret) = SQL_FETCHROW($result);
		} else {
			// Return title and price
			list($t, $p) = SQL_FETCHROW($result);
			$ret = $t." / ".TRANSLATE_COMMA($p)." ".POINTS;
		}
	}

	// Free result
	SQL_FREERESULT($result);

	// Return result
	return $ret;
}

// Get (basicly) the price of given payment id
function GET_PAY_POINTS($pid, $lookFor="price")
{
	$ret = "-1";
	$result = SQL_QUERY_ESC("SELECT %s FROM "._MYSQL_PREFIX."_payments WHERE id=%s LIMIT 1",
		array($lookFor, $pid), __FILE__, __LINE__);
	if (SQL_NUMROWS($result) == 1)
	{
		// Payment type found... :-)
		list($ret) = SQL_FETCHROW($result);
		SQL_FREERESULT($result);
	}
	return $ret;
}

// Remove a receiver's ID from $ARRAY and add a link for him to confirm
function REMOVE_RECEIVER (&$ARRAY, $key, $uid, $pool_id, $stats_id="", $bonus=false)
{
	$ret = "failed";
	if ($uid > 0)
	{
		// Remove entry from array
		unset($ARRAY[$key]);

		// Is there already a line for this user available?
		if ($stats_id > 0)
		{
			// Only when we got a real stats ID continue searching for the entry
			$type = "NORMAL"; $rowName = "stats_id";
			if ($bonus) { $type = "BONUS"; $rowName = "bonus_id"; }
			$result = SQL_QUERY_ESC("SELECT id FROM "._MYSQL_PREFIX."_user_links WHERE %s='%s' AND userid=%s AND link_type='%s' LIMIT 1",
			 array($rowName, $stats_id, bigintval($uid), $type), __FILE__, __LINE__);
			if (SQL_NUMROWS($result) == 0)
			{
				// No, so we add one!
				$result = SQL_QUERY_ESC("INSERT INTO "._MYSQL_PREFIX."_user_links (%s, userid, link_type) VALUES ('%s','%s','%s')",
				 array($rowName, $stats_id, bigintval($uid), $type), __FILE__, __LINE__);
				$ret = "done";
			}
			 else
			{
				// Already found
				$ret = "already";
			}

			// Free memory
			SQL_FREERESULT($result);
		}
	}
	// Return status for sending routine
	return $ret;
}

// Calculate sum (default) or count records of given criteria
function GET_TOTAL_DATA ($search, $tableName, $lookFor, $whereStatement="userid", $onlyRows=false, $add="") {
	$ret = 0;
	//* DEBUG: */ echo $search."/".$tableName."/".$lookFor."/".$whereStatement."/".$add."<br />\n";
	if (($onlyRows) || ($lookFor == "userid")) {
		// Count rows
		//* DEBUG: */ echo "COUNT!<br />\n";
		$result = SQL_QUERY_ESC("SELECT COUNT(`%s`) FROM `"._MYSQL_PREFIX."_%s` WHERE `%s`='%s'".$add,
			array($lookFor, $tableName, $whereStatement, $search), __FILE__, __LINE__);
	} else {
		// Add all rows
		//* DEBUG: */ echo "SUM!<br />\n";
		$result = SQL_QUERY_ESC("SELECT SUM(`%s`) FROM `"._MYSQL_PREFIX."_%s` WHERE `%s`='%s'".$add,
			array($lookFor, $tableName, $whereStatement, $search), __FILE__, __LINE__);
	}

	// Load row
	list($ret) = SQL_FETCHROW($result);

	// Free result
	SQL_FREERESULT($result);

	// Fix empty values
	if ((empty($ret)) && ($lookFor != "counter") && ($lookFor != "id") && ($lookFor != "userid")) {
		// Float number
		$ret = "0.00000";
	} elseif ("".$ret."" == "") {
		// Fix empty result
		$ret = "0";
	}

	// Return value
	return $ret;
}
// "Getter fro ref level percents
function GET_REF_LEVEL_PERCENTS ($level) {
	global $cacheInstance, $_CONFIG, $cacheArray;

	// Default is zero
	$per = 0;

	// Do we have cache?
	if ((isset($cacheArray['ref_depths']['level'])) && (EXT_IS_ACTIVE("cache"))) {
		// First look for level
		$key = array_search($level, $cacheArray['ref_depths']['level']);
		if ($key !== false) {
			// Entry found!
			$per = $cacheArray['ref_depths']['percents'][$key];

			// Count cache hit
			if (getConfig('cache_hits') > 0) { $_CONFIG['cache_hits']++; } else { $_CONFIG['cache_hits'] = 1; }
		}
	} elseif (!EXT_IS_ACTIVE("cache")) {
		// Get referal data
		$result_lvl = SQL_QUERY_ESC("SELECT percents FROM "._MYSQL_PREFIX."_refdepths WHERE level='%s' LIMIT 1",
			array(bigintval($level)), __FILE__, __LINE__);

		// Entry found?
		if (SQL_NUMROWS($result_lvl) == 1) {
			// Get percents
			list($per) = SQL_FETCHROW($result_lvl);
		} // END - if

		// Free result
		SQL_FREERESULT($result_lvl);
	}

	// Return percent
	return $per;
}
/**
 *
 * Dynamic referal system, can also send mails!
 *
 * subject     = Subject line, write in lower-case letters and underscore is allowed
 * uid         = Referal ID wich should receive...
 * points      = ... xxx points
 * send_notify = shall I send the referal an email or not?
 * rid         = inc/modules/guest/what-confirm.php need this (DEPRECATED???)
 * locked      = Shall I pay it to normal (false) or locked (true) points ammount?
 * add_mode    = Add points only to $uid or also refs? (WARNING! Changing "ref" to "direct"
 *               for default value will cause no referal will get points ever!!!)
 */
function ADD_POINTS_REFSYSTEM ($subject, $uid, $points, $send_notify=false, $rid="0", $locked=false, $add_mode="ref") {
	//* DEBUG: */ print "----------------------- <font color=\"#00aa00\">".__FUNCTION__." - ENTRY</font> ------------------------<ul><li>\n";
	global $_CONFIG, $DATA, $cacheArray;

	// Convert mode to lower-case
	$add_mode = strtolower($add_mode);

	// When $uid = 0 add points to jackpot
	if ($uid == "0") {
		// Add points to jackpot
		ADD_JACKPOT($points);
		return;
	} // END - if

	// Add booking record if extension is installed
	if (EXT_IS_ACTIVE("booking")) {
		// Add record
		ADD_BOOKING_RECORD($subject, $uid, $points, "add");
	} // END - if

	// Count up referal depth
	if (!isset($GLOBALS['ref_level'])) {
		// Initialialize referal system
		//* DEBUG: */ print __FUNCTION__."(<font color=\"#0000aa\">".__LINE__."</font>): Referal system initialized!<br />\n";
		$GLOBALS['ref_level'] = 0;
	} else {
		// Increase referal level
		$GLOBALS['ref_level']++;
		//* DEBUG: */ print __FUNCTION__."(<font color=\"#0000aa\">".__LINE__."</font>): Referal level increased. DEPTH={$GLOBALS['ref_level']}<br />\n";
	}

	// Default is "normal" points
	$data = "points";

	// Which points, locked or normal?
	if ($locked) $data = "locked_points";

	// Check user account
	$result_user = SQL_QUERY_ESC("SELECT refid, email FROM `"._MYSQL_PREFIX."_user_data` WHERE userid=%s AND status='CONFIRMED' LIMIT 1",
		array(bigintval($uid)), __FILE__, __LINE__);

	//* DEBUG: */ print __FUNCTION__."(<font color=\"#0000aa\">".__LINE__."</font>):uid={$uid},numRows=".SQL_NUMROWS($result_user).",points={$points}<br />\n";
	if (SQL_NUMROWS($result_user) == 1) {
		// This is the user and his ref
		list($ref, $email) = SQL_FETCHROW($result_user);
		$cacheArray['add_uid'][$ref] = $uid;

		// Get percents
		$per = GET_REF_LEVEL_PERCENTS($GLOBALS['ref_level']);
		//* DEBUG: */ print __FUNCTION__."(<font color=\"#0000aa\">".__LINE__."</font>):uid={$uid},points={$points},depth={$GLOBALS['ref_level']},per={$per},mode={$add_mode}<br />\n";

		// Some percents found?
		if ($per > 0) {
			// Calculate new points
			//* DEBUG: */ print __FUNCTION__."(<font color=\"#0000aa\">".__LINE__."</font>):uid={$uid},points={$points},per={$per},depth={$GLOBALS['ref_level']}<br />\n";
			$ref_points = $points * $per / 100;

			// Pay refback here if level > 0 and in ref-mode
			if ((EXT_IS_ACTIVE("refback")) && ($GLOBALS['ref_level'] > 0) && ($per < 100) && ($add_mode == "ref") && (isset($cacheArray['add_uid'][$uid]))) {
				//* DEBUG: */ print __FUNCTION__."(<font color=\"#0000aa\">".__LINE__."</font>):uid={$uid},data={$cacheArray['add_uid'][$uid]},ref_points={$ref_points},depth={$GLOBALS['ref_level']} - BEFORE!<br />\n";
				$ref_points = ADD_REFBACK_POINTS($cacheArray['add_uid'][$uid], $uid, $points, $ref_points);
				//* DEBUG: */ print __FUNCTION__."(<font color=\"#0000aa\">".__LINE__."</font>):uid={$uid},data={$cacheArray['add_uid'][$uid]},ref_points={$ref_points},depth={$GLOBALS['ref_level']} - AFTER!<br />\n";
			} // END - if

			// Update points...
			SQL_QUERY_ESC("UPDATE "._MYSQL_PREFIX."_user_points SET %s=%s+%s WHERE userid=%s AND ref_depth='%s' LIMIT 1",
				array($data, $data, $ref_points, bigintval($uid), bigintval($GLOBALS['ref_level'])), __FILE__, __LINE__);
			//* DEBUG: */ print __FUNCTION__."(<font color=\"#0000aa\">".__LINE__."</font>):data={$data},ref_points={$ref_points},uid={$uid},depth={$GLOBALS['ref_level']},mode={$add_mode} - UPDATE! (".SQL_AFFECTEDROWS().")<br />\n";

			// No entry updated?
			if (SQL_AFFECTEDROWS() < 1) {
				// First ref in this level! :-)
				$result = SQL_QUERY_ESC("INSERT INTO "._MYSQL_PREFIX."_user_points (userid,ref_depth,%s) VALUES (%s,%s,%s)",
					array($data, bigintval($uid), bigintval($GLOBALS['ref_level']), $ref_points), __FILE__, __LINE__);
				//* DEBUG: */ print __FUNCTION__."(<font color=\"#0000aa\">".__LINE__."</font>):data={$data},ref_points={$ref_points},uid={$uid},depth={$GLOBALS['ref_level']},mode={$add_mode} - INSERTED! (".SQL_AFFECTEDROWS().")<br />\n";
			} // END - if

			// Update mediadata as well
			if (GET_EXT_VERSION("mediadata") >= "0.0.4") {
				// Update database
				MEDIA_UPDATE_ENTRY(array("total_points"), "add", $ref_points);
			} // END - if

			// Points updated, maybe I shall send him an email?
			if (($send_notify) && ($ref > 0) && (!$locked)) {
				// Prepare content
				$content = array(
					'percent' => $per,
					'level'   => bigintval($GLOBALS['ref_level']),
					'points'  => $ref_points,
					'refid'   => bigintval($ref)
				);

				// Load email template
				$msg = LOAD_EMAIL_TEMPLATE("confirm-referal", $content, bigintval($uid));

				SEND_EMAIL($email, THANX_REFERRAL_ONE, $msg);
			} elseif (($send_notify) && ($ref == 0) && (!$locked) && ($add_mode == "direct") && (!defined('__POINTS_VALUE'))) {
				// Direct payment shall be notified about
				define('__POINTS_VALUE', $ref_points);

				// Prepare content
				$content = array(
					'text'   => REASON_DIRECT_PAYMENT,
					'points' => TRANSLATE_COMMA($ref_points)
				);

				// Load message
				$msg = LOAD_EMAIL_TEMPLATE("add-points", $content, $uid);

				// And sent it away
				SEND_EMAIL($email, SUBJECT_DIRECT_PAYMENT, $msg);
				if (!isset($_GET['mid'])) LOAD_TEMPLATE("admin_settings_saved", false, ADMIN_POINTS_ADDED);
			}

			// Maybe there's another ref?
			if (($ref > 0) && ($points > 0) && ($ref != $uid) && ($add_mode == "ref")) {
				// Then let's credit him here...
				//* DEBUG: */ print __FUNCTION__."(<font color=\"#0000aa\">".__LINE__."</font>):uid={$uid},ref={$ref},points={$points} - ADVANCE!<br />\n";
				ADD_POINTS_REFSYSTEM(sprintf("%s_ref:%s", $subject, $GLOBALS['ref_level']), $ref, $points, $send_notify, $ref, $locked);
			} // END - if
		} // END - if
	} // END - if

	// Free result
	SQL_FREERESULT($result_user);
	//* DEBUG: */ print "</li></ul>----------------------- <font color=\"#aa0000\">".__FUNCTION__." - EXIT</font> ------------------------<br />\n";
}
//
function UPDATE_REF_COUNTER ($uid) {
	global $cacheArray, $cacheInstance;

	// Make it sure referal level zero (member him-/herself) is at least selected
	if (empty($cacheArray['ref_level'][$uid])) $cacheArray['ref_level'][$uid] = 1;
	//* DEBUG: */ print __FUNCTION__."(<font color=\"#0000aa\">".__LINE__."</font>):uid={$uid},level={$cacheArray['ref_level'][$uid]}<br />\n";

	// Update counter
	$result = SQL_QUERY_ESC("UPDATE "._MYSQL_PREFIX."_refsystem SET counter=counter+1 WHERE userid=%s AND level='%s' LIMIT 1",
		array(bigintval($uid), $cacheArray['ref_level'][$uid]), __FILE__, __LINE__);

	// When no entry was updated then we have to create it here
	//* DEBUG: */ print __FUNCTION__."(<font color=\"#0000aa\">".__LINE__."</font>):updated=".SQL_AFFECTEDROWS()."<br />\n";
	if (SQL_AFFECTEDROWS() < 1) {
		// First count!
		$result = SQL_QUERY_ESC("INSERT INTO "._MYSQL_PREFIX."_refsystem (userid, level, counter) VALUES (%s,%s,1)",
			array(bigintval($uid), $cacheArray['ref_level'][$uid]), __FILE__, __LINE__);
		//* DEBUG: */ print __FUNCTION__."(<font color=\"#0000aa\">".__LINE__."</font>):uid={$uid}<br />\n";
	} // END - if

	// Check for his referal
	$result = SQL_QUERY_ESC("SELECT refid FROM `"._MYSQL_PREFIX."_user_data` WHERE userid=%s LIMIT 1",
		array(bigintval($uid)), __FILE__, __LINE__);

	// Load refid
	list($ref) = SQL_FETCHROW($result);

	// Free memory
	SQL_FREERESULT($result);
	//* DEBUG: */ print __FUNCTION__."(<font color=\"#0000aa\">".__LINE__."</font>):uid={$uid},ref={$ref}<br />\n";

	// When he has a referal...
	if (($ref > 0) && ($ref != $uid)) {
		// Move to next referal level and count his counter one up!
		//* DEBUG: */ print __FUNCTION__."(<font color=\"#0000aa\">".__LINE__."</font>):ref={$ref} - ADVANCE!<br />\n";
		$cacheArray['ref_level'][$uid]++; UPDATE_REF_COUNTER($ref);
	} elseif ((($ref == $uid) || ($ref == 0)) && (GET_EXT_VERSION("cache") >= "0.1.2")) {
		// Remove cache here
		//* DEBUG: */ print __FUNCTION__."(<font color=\"#0000aa\">".__LINE__."</font>):ref={$ref} - CACHE!<br />\n";
		REBUILD_CACHE("refsystem", "refsystem");
	}

	// "Walk" back here
	$cacheArray['ref_level'][$uid]--;

	// Handle refback here if extension is installed
	if (EXT_IS_ACTIVE("refback")) {
		UPDATE_REFBACK_TABLE($uid);
	} // END - if
}

// OBSOLETE: Sends out mail to all administrators
function SEND_ADMIN_EMAILS ($subj, $msg) {
	// Load all admin email addresses
	$result = SQL_QUERY("SELECT email FROM "._MYSQL_PREFIX."_admins ORDER BY id ASC", __FILE__, __LINE__);
	while (list($email) = SQL_FETCHROW($result)) {
		// Send the email out
		SEND_EMAIL($email, $subj, $msg);
	} // END - if

	// Free result
	SQL_FREERESULT($result);

	// Really simple... ;-)
}

// Get ID number from administrator's login name
function GET_ADMIN_ID ($login) {
	global $cacheArray, $_CONFIG;
	$ret = "-1";
	if (isset($cacheArray['admins']['aid'][$login])) {
		// Check cache
		$ret = $cacheArray['admins']['aid'][$login];

		// Update cache hits
		if (getConfig('cache_hits') > 0) { $_CONFIG['cache_hits']++; } else { $_CONFIG['cache_hits'] = 1; }
	} elseif (!EXT_IS_ACTIVE("cache")) {
		// Load from database
		$result = SQL_QUERY_ESC("SELECT id FROM "._MYSQL_PREFIX."_admins WHERE login='%s' LIMIT 1",
		 array($login), __FILE__, __LINE__);
		if (SQL_NUMROWS($result) == 1) {
			list($ret) = SQL_FETCHROW($result);
		} // END - if

		// Free result
		SQL_FREERESULT($result);
	}
	return $ret;
}

// "Getter" for current admin id
function GET_CURRENT_ADMIN_ID () {
	// Get the admin login from session
	$adminLogin = get_session('admin_login');

	// "Solve" it into an id
	$adminId = GET_ADMIN_ID($adminLogin);

	// Secure and return it
	return bigintval($adminId);
}

// Get password hash from administrator's login name
function GET_ADMIN_HASH ($aid)
{
	global $cacheArray, $_CONFIG;
	$ret = "-1";
	if (isset($cacheArray['admins']['password'][$aid])) {
		// Check cache
		$ret = $cacheArray['admins']['password'][$aid];

		// Update cache hits
		if (getConfig('cache_hits') > 0) { $_CONFIG['cache_hits']++; } else { $_CONFIG['cache_hits'] = 1; }
	} elseif (!EXT_IS_ACTIVE("cache")) {
		// Load from database
		$result = SQL_QUERY_ESC("SELECT password FROM "._MYSQL_PREFIX."_admins WHERE id=%s LIMIT 1",
		 array($aid), __FILE__, __LINE__);
		if (SQL_NUMROWS($result) == 1) {
			// Fetch data
			list($ret) = SQL_FETCHROW($result);

			// Set cache
			$cacheArray['admins']['password'][$aid] = $ret;
		}

		// Free result
		SQL_FREERESULT($result);
	}
	return $ret;
}
//
function GET_ADMIN_LOGIN ($aid) {
	global $cacheArray, $_CONFIG;
	$ret = "***";
	if (isset($cacheArray['admins']['login'][$aid])) {
		// Get cache
		$ret = $cacheArray['admins']['login'][$aid];

		// Update cache hits
		if (getConfig('cache_hits') > 0) { $_CONFIG['cache_hits']++; } else { $_CONFIG['cache_hits'] = 1; }
	} elseif (!EXT_IS_ACTIVE("cache")) {
		// Load from database
		$result = SQL_QUERY_ESC("SELECT login FROM "._MYSQL_PREFIX."_admins WHERE id=%s LIMIT 1",
			array(bigintval($aid)), __FILE__, __LINE__);
		if (SQL_NUMROWS($result) == 1) {
			// Fetch data
			list($ret) = SQL_FETCHROW($result);

			// Set cache
			$cacheArray['admins']['login'][$aid] = $ret;
		} // END - if

		// Free memory
		SQL_FREERESULT($result);
	}
	return $ret;
}
// Get email address of admin id
function GET_ADMIN_EMAIL ($aid) {
	global $cacheArray, $_CONFIG;

	$ret = "***";
	if (isset($cacheArray['admins']['email'][$aid])) {
		// Get cache
		$ret = $cacheArray['admins']['email'][$aid];

		// Update cache hits
		if (getConfig('cache_hits') > 0) { $_CONFIG['cache_hits']++; } else { $_CONFIG['cache_hits'] = 1; }
	} elseif (!EXT_IS_ACTIVE("cache")) {
		// Load from database
		$result_aid = SQL_QUERY_ESC("SELECT email FROM "._MYSQL_PREFIX."_admins WHERE id=%s LIMIT 1",
			array(bigintval($aid)), __FILE__, __LINE__);
		if (SQL_NUMROWS($result_aid) == 1) {
			// Get data
			list($ret) = SQL_FETCHROW($result_aid);

			// Set cache
			$cacheArray['admins']['email'][$aid] = $ret;
			} // END - if

		// Free result
		SQL_FREERESULT($result_aid);
	}

	// Return email
	return $ret;
}
// Get default ACL  of admin id
function GET_ADMIN_DEFAULT_ACL ($aid) {
	global $cacheArray, $_CONFIG;

	$ret = "***";
	if (isset($cacheArray['admins']['def_acl'][$aid])) {
		// Use cache
		$ret = $cacheArray['admins']['def_acl'][$aid];

		// Update cache hits
		if (getConfig('cache_hits') > 0) { $_CONFIG['cache_hits']++; } else { $_CONFIG['cache_hits'] = 1; }
	} elseif (!EXT_IS_ACTIVE("cache")) {
		// Load from database
		$result_aid = SQL_QUERY_ESC("SELECT default_acl FROM "._MYSQL_PREFIX."_admins WHERE id=%s LIMIT 1",
			array(bigintval($aid)), __FILE__, __LINE__);
		if (SQL_NUMROWS($result_aid) == 1) {
			// Fetch data
			list($ret) = SQL_FETCHROW($result_aid);

			// Set cache
			$cacheArray['admins']['def_acl'][$aid] = $ret;
		}

		// Free result
		SQL_FREERESULT($result_aid);
	}

	// Return email
	return $ret;
}
//
function ADD_OPTION_LINES($table, $id, $name, $default="",$special="",$where="") {
	$ret = "";
	if ($table == "/ARRAY/") {
		// Selection from array
		if (is_array($id) && is_array($name) && sizeof($id) == sizeof($name)) {
			// Both are arrays
			foreach ($id as $idx => $value) {
				$ret .= "<OPTION value=\"".$value."\"";
				if ($default == $value) $ret .= " selected checked";
				$ret .= ">".$name[$idx]."</OPTION>\n";
			} // END - foreach
		} // END - if
	} else {
		// Data from database
		$SPEC = ", ".$id;
		if (!empty($special)) $SPEC = ", ".$special;
		$ORDER = $name.$SPEC;
		if ($table == "country") $ORDER = $special;
		$result = SQL_QUERY_ESC("SELECT %s, %s".$SPEC." FROM "._MYSQL_PREFIX."_%s ".$where." ORDER BY %s",
		 array($id, $ORDER, $table, $name), __FILE__, __LINE__);
		if (SQL_NUMROWS($result) > 0) {
			// Found data so add them as OPTION lines: $id is the value and $name is the "name" of the option
			while (list($value, $title, $add) = SQL_FETCHROW($result)) {
				if (empty($special)) $add = "";
				$ret .= "<OPTION value=\"".$value."\"";
				if ($default == $value) $ret .= " selected checked";
				if (!empty($add)) $add = " (".$add.")";
				$ret .= ">".$title.$add."</OPTION>\n";
			} // END - while

			// Free memory
			SQL_FREERESULT($result);
		} else {
			// No data found
			$ret = "<OPTION value=\"x\">".SELECT_NONE."</OPTION>\n";
		}
	}

	// Return - hopefully - the requested data
	return $ret;
}
// Activate exchange (DEPERECATED???)
function activateExchange() {
	global $_CONFIG;
	$result = SQL_QUERY("SELECT userid FROM `"._MYSQL_PREFIX."_user_data` WHERE status='CONFIRMED' AND max_mails > 0", __FILE__, __LINE__);
	if (SQL_NUMROWS($result) >= getConfig('activate_xchange'))
	{
		// Free memory
		SQL_FREERESULT($result);

		// Activate System
		$SQLs = array(
			"UPDATE "._MYSQL_PREFIX."_mod_reg SET locked='N', hidden='N', mem_only='Y' WHERE module='order' LIMIT 1",
			"UPDATE `"._MYSQL_PREFIX."_member_menu` SET visible='Y', locked='N' WHERE what='order' OR what='unconfirmed' LIMIT 2",
			"UPDATE `"._MYSQL_PREFIX."_config` SET activate_xchange='0' WHERE config=0 LIMIT 1"
		);

		// Run SQLs
		foreach ($SQLs as $sql) {
			$result = SQL_QUERY($sql, __FILE__, __LINE__);
		}

		// @TODO Destroy cache
	}
}
//
function DELETE_USER_ACCOUNT($uid, $reason)
{
	$points = 0;
	$result = SQL_QUERY_ESC("SELECT (SUM(p.points) - d.used_points) AS points
FROM "._MYSQL_PREFIX."_user_points AS p
LEFT JOIN `"._MYSQL_PREFIX."_user_data` AS d
ON p.userid=d.userid
WHERE p.userid=%s", array(bigintval($uid)), __FILE__, __LINE__);
	if (SQL_NUMROWS($result) == 1) {
		// Save his points to add them to the jackpot
		list($points) = SQL_FETCHROW($result);
		SQL_FREERESULT($result);

		// Delete points entries as well
		$result = SQL_QUERY_ESC("DELETE LOW_PRIORITY FROM "._MYSQL_PREFIX."_user_points WHERE userid=%s", array(bigintval($uid)), __FILE__, __LINE__);

		// Update mediadata as well
		if (GET_EXT_VERSION("mediadata") >= "0.0.4") {
			// Update database
			MEDIA_UPDATE_ENTRY(array("total_points"), "sub", $points);
		} // END - if

		// Now, when we have all his points adds them do the jackpot!
		ADD_JACKPOT($points);
	}

	// Delete category selections as well...
	$result = SQL_QUERY_ESC("DELETE LOW_PRIORITY FROM "._MYSQL_PREFIX."_user_cats WHERE userid=%s",
	 array(bigintval($uid)), __FILE__, __LINE__);

	// Remove from rallye if found
	if (EXT_IS_ACTIVE("rallye")) {
		$result = SQL_QUERY_ESC("DELETE LOW_PRIORITY FROM "._MYSQL_PREFIX."_rallye_users WHERE userid=%s",
		 array(bigintval($uid)), __FILE__, __LINE__);
	}

	// Now a mail to the user and that's all...
	$msg = LOAD_EMAIL_TEMPLATE("del-user", array('text' => $reason), $uid);
	SEND_EMAIL($uid, ADMIN_DEL_ACCOUNT, $msg);

	// Ok, delete the account!
	$result = SQL_QUERY_ESC("DELETE LOW_PRIORITY FROM `"._MYSQL_PREFIX."_user_data` WHERE userid=%s LIMIT 1", array(bigintval($uid)), __FILE__, __LINE__);
}
//
function META_DESCRIPTION ($mod, $wht) {
	global $_CONFIG;

	// Exclude admin and member's area
	if (($mod != "admin") && ($mod != "login")) {
		// Construct dynamic description
		$DESCR = MAIN_TITLE." ".trim(getConfig('title_middle'))." ".ADD_DESCR("guest", "what-".$wht, true);

		// Output it directly
		OUTPUT_HTML("<meta name=\"description\" content=\"".$DESCR."\" />");
	} // END - if

	// Remove depth
	unset($GLOBALS['ref_level']);
}
//
function ADD_JACKPOT($points) {
	$result = SQL_QUERY("SELECT points FROM "._MYSQL_PREFIX."_jackpot WHERE ok='ok' LIMIT 1", __FILE__, __LINE__);
	if (SQL_NUMROWS($result) == 0) {
		// Create line
		$result = SQL_QUERY_ESC("INSERT INTO "._MYSQL_PREFIX."_jackpot (ok, points) VALUES ('ok','%s')", array($points), __FILE__, __LINE__);
	} else {
		// Free memory
		SQL_FREERESULT($result);

		// Update points
		$result = SQL_QUERY_ESC("UPDATE "._MYSQL_PREFIX."_jackpot SET points=points+%s WHERE ok='ok' LIMIT 1",
			array($points), __FILE__, __LINE__);
	}
}
//
function SUB_JACKPOT($points) {
	// First failed
	$ret = "-1";

	// Get current points
	$result = SQL_QUERY("SELECT points FROM "._MYSQL_PREFIX."_jackpot WHERE ok='ok' LIMIT 1", __FILE__, __LINE__);
	if (SQL_NUMROWS($result) == 0) {
		// Create line
		SQL_QUERY("INSERT INTO "._MYSQL_PREFIX."_jackpot (ok, points) VALUES ('ok', 0.00000)", __FILE__, __LINE__);
	} else {
		// Read points
		list($jackpot) = SQL_FETCHROW($result);
		if ($jackpot >= $points) {
			// Update points when there are enougth points in jackpot
			SQL_QUERY_ESC("UPDATE "._MYSQL_PREFIX."_jackpot SET points=points-%s WHERE ok='ok' LIMIT 1",
				array($points), __FILE__, __LINE__);
			$ret = $jackpot - $points;
		} // END - if
	}

	// Free memory
	SQL_FREERESULT($result);
}
//
function IS_DEMO() {
	return ((EXT_IS_ACTIVE("demo")) && (get_session('admin_login') == "demo"));
}
//
function LOAD_CONFIG ($no="0") {
	global $cacheArray;
	$CFG_DUMMY = array();

	// Check for cache extension, cache-array and if the requested configuration is in cache
	if ((is_array($cacheArray)) && (isset($cacheArray['config'][$no])) && (is_array($cacheArray['config'][$no]))) {
		// Load config from cache
		//* DEBUG: */ echo gettype($cacheArray['config'][$no])."<br />\n";
		foreach ($cacheArray['config'][$no] as $key => $value) {
			$CFG_DUMMY[$key] = $value;
		} // END - foreach

		// Count cache hits if exists
		if ((isset($CFG_DUMMY['cache_hits'])) && (EXT_IS_ACTIVE("cache"))) {
			$CFG_DUMMY['cache_hits']++;
		} // END - if
	} elseif ((!EXT_IS_ACTIVE("cache")) || (!isset($cacheArray['config'][$no]))) {
		// Load config from DB
		$result_config = SQL_QUERY_ESC("SELECT * FROM `"._MYSQL_PREFIX."_config` WHERE config=%d LIMIT 1",
			array(bigintval($no)), __FILE__, __LINE__);

		// Get config from database
		$CFG_DUMMY = SQL_FETCHARRAY($result_config);

		// Free result
		SQL_FREERESULT($result_config);

		// Remember this config in the array
		$cacheArray['config'][$no] = $CFG_DUMMY;
	}

	// Return config array
	return $CFG_DUMMY;
}
// Gets the matching what name from module
function GET_WHAT($modCheck) {
	global $_CONFIG;

	$wht = "";
	//* DEBUG: */ echo __LINE__."!".$modCheck."!<br />\n";
	switch ($modCheck)
	{
	case "admin":
		$wht = "overview";
		break;

	case "login":
	case "index":
		$wht = "welcome";
		if (($modCheck == "index") && (getConfig('index_home') != "")) $wht = getConfig('index_home');
		break;

	default:
		$wht = "";
		break;
	}

	// Return what value
	return $wht;
}

// Subtract points from database and mediadata cache
function SUB_POINTS ($subject, $uid, $points) {
	// Add points to used points
	$result = SQL_QUERY_ESC("UPDATE `"._MYSQL_PREFIX."_user_data` SET `used_points`=`used_points`+%s WHERE userid=%s LIMIT 1",
	 array($points, bigintval($uid)), __FILE__, __LINE__);

	// Insert booking record
	if (EXT_IS_ACTIVE("booking")) {
		// Add record
		ADD_BOOKING_RECORD($subject, $uid, $points, "sub");
	} // END - if

	// Update mediadata as well
	if (GET_EXT_VERSION("mediadata") >= "0.0.4") {
		// Update database
		MEDIA_UPDATE_ENTRY(array("total_points"), "sub", $points);
	} // END - if
}

// Update config entries
function UPDATE_CONFIG ($entries, $values, $updateMode="") {
	global $CSS;

	// Do not update config in CSS mode
	if (($CSS == "1") || ($CSS == -1)) {
		return;
	} // END - if

	// Do we have multiple entries?
	if (is_array($entries)) {
		// Walk through all
		$all = "";
		foreach ($entries as $idx => $entry) {
			// Update mode set?
			if (!empty($updateMode)) {
				// Update entry
				$all .= sprintf("%s=%s%s%s,", $entry, $entry, $updateMode, (float)$values[$idx]);
			} else {
				// Check if string or number
				if (($values[$idx] + 0) === $values[$idx]) {
					// Number detected
					$all .= sprintf("%s=%s,", $entry, (float)$values[$idx]);
				} elseif ($values[$idx] == "UNIX_TIMESTAMP()") {
					// Function UNIX_TIMESTAMP() detected
					$all .= sprintf("%s=%s,", $entry, $values[$idx]);
				} else {
					// String detected
					$all .= sprintf("%s='%s',", $entry, SQL_ESCAPE($values[$idx]));
				}
			}
		} // END - foreach

		// Remove last comma
		$entries = substr($all, 0, -1);
	} elseif (!empty($updateMode)) {
		// Update mode set
		$entries .= sprintf("=%s%s%s", $entries, $updateMode, (float)$values);
	} else {
		// Regular entry to update
		$entries .= sprintf("='%s'", SQL_ESCAPE($values));
	}

	// Run database update
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "entries={$entries}");
	SQL_QUERY("UPDATE `"._MYSQL_PREFIX."_config` SET ".$entries." WHERE config=0 LIMIT 1", __FILE__, __LINE__);

	// Get affected rows
	$affectedRows = SQL_AFFECTEDROWS();
	//* DEBUG: */ print __FUNCTION__."(<font color=\"#0000aa\">".__LINE__."</font>):entries={$entries},affectedRows={$affectedRows}<br />\n";

	// Rebuild cache
	REBUILD_CACHE("config", "config");
}

// Prepares an SQL statement part for HTML mail and/or holiday depency
function PREPARE_SQL_HTML_HOLIDAY ($mode) {
	// Exclude no users by default
	$MORE = "";

	// HTML mail?
	if ($mode == "html") $MORE = " AND html='Y'";
	if (GET_EXT_VERSION("holiday") >= "0.1.3") {
		// Add something for the holiday extension
		$MORE .= " AND holiday_active='N'";
	} // END - if

	// Return result
	return $MORE;
}

// "Getter" for total available receivers
function GET_TOTAL_RECEIVERS ($mode="normal") {
	// Query database
	$result_all = SQL_QUERY("SELECT userid
FROM "._MYSQL_PREFIX."_user_data
WHERE status='CONFIRMED' AND receive_mails > 0".PREPARE_SQL_HTML_HOLIDAY($mode),
		__FILE__, __LINE__);

	// Get num rows
	$numRows = SQL_NUMROWS($result_all);

	// Free result
	SQL_FREERESULT($result_all);

	// Return value
	return $numRows;
}

// Returns HTML code with an "<option> list" of all categories
function ADD_CATEGORY_OPTIONS ($mode) {
	// Prepare WHERE statement
	$whereStatement = " WHERE visible='Y'";
	if (IS_ADMIN()) $whereStatement = "";

	// Initialize array...
	$CATS = array(
		'id'   => array(),
		'name' => array(),
		'uids' => array()
	);

	// Get categories
	$result = SQL_QUERY("SELECT id, cat FROM "._MYSQL_PREFIX."_cats".$whereStatement." ORDER BY sort", __FILE__, __LINE__);
	if (SQL_NUMROWS($result) > 0) {
		// ... and begin loading stuff
		while (list($id, $cat) = SQL_FETCHROW($result)) {
			// Transfer some data
			$CATS['id'][]   = $id;
			$CATS['name'][] = $cat;

			// Check which users are in this category
			$result_uids = SQL_QUERY_ESC("SELECT userid FROM "._MYSQL_PREFIX."_user_cats WHERE cat_id=%s",
			 array(bigintval($id)), __FILE__, __LINE__);

			// Start adding all
			$uid_cnt = 0;
			while (list($ucat) = SQL_FETCHROW($result_uids)) {
				$result_ver = SQL_QUERY_ESC("SELECT userid FROM "._MYSQL_PREFIX."_user_data
WHERE userid=%s AND status='CONFIRMED' AND receive_mails > 0".PREPARE_SQL_HTML_HOLIDAY($mode)." LIMIT 1",
 array(bigintval($ucat)), __FILE__, __LINE__);
				$uid_cnt += SQL_NUMROWS($result_ver);

				// Free memory
				SQL_FREERESULT($result_ver);
			} // END - while

			// Free memory
			SQL_FREERESULT($result_uids);

			// Add counter
			$CATS['uids'][] = $uid_cnt;
		}

		// Free memory
		SQL_FREERESULT($result);

		// Generate options
		$OUT = "";
		foreach ($CATS['id'] as $key => $value) {
			if (strlen($CATS['name'][$key]) > 20) $CATS['name'][$key] = substr($CATS['name'][$key], 0, 17)."...";
			$OUT .= "      <OPTION value=\"".$value."\">".$CATS['name'][$key]." (".$CATS['uids'][$key]." ".USER_IN_CAT.")</OPTION>\n";
		}
	} else {
		// No cateogries are defined yet
		$OUT = "<option class=\"member_failed\">".MEMBER_NO_CATS."</option>\n";
	}

	// Return HTML code
	return $OUT;
}

// Add bonus mail to queue
function ADD_BONUS_MAIL_TO_QUEUE ($subject, $text, $receiverList, $points, $seconds, $url, $cat, $mode="normal", $receiver=0) {
	// Is admin or bonus extension there?
	if (!IS_ADMIN()) {
		// Abort here
		return false;
	} elseif (!EXT_IS_ACTIVE("bonus")) {
		// Abort here
		return false;
	}

	// Calculcate target sent
	$target = SELECTION_COUNT(explode(";", $receiverList));

	// Receiver is zero?
	if ($receiver == 0) {
		// Then auto-fix it
		$receiver = $target;
	} // END - if

	// HTML extension active?
	if (EXT_IS_ACTIVE("html")) {
		// No HTML by default
		$HTML = "N";

		// HTML mode?
		if ($mode == "html") $HTML = "Y";

		// Add HTML mail
		$result = SQL_QUERY_ESC("INSERT INTO "._MYSQL_PREFIX."_bonus
(subject, text, receivers, points, time, data_type, timestamp, url, cat_id, target_send, mails_sent, html_msg)
VALUES ('%s','%s','%s','%s','%s','NEW', UNIX_TIMESTAMP(),'%s','%s','%s','%s','%s')",
 array(
	$subject,
	$text,
	$receiverList,
	$points,
	$seconds,
	$url,
	$cat,
	$target,
	bigintval($receiver),
	$HTML
), __FILE__, __LINE__);
	} else {
		// Add regular mail
		$result = SQL_QUERY_ESC("INSERT INTO "._MYSQL_PREFIX."_bonus
(subject, text, receivers, points, time, data_type, timestamp, url, cat_id, target_send, mails_sent)
VALUES ('%s','%s','%s','%s','%s','NEW', UNIX_TIMESTAMP(),'%s','%s','%s','%s')",
 array(
	$subject,
	$text,
	$receiverList,
	$points,
	$seconds,
	$url,
	$cat,
	$target,
	bigintval($receiver),
), __FILE__, __LINE__);
	}
}

// Generate a receiver list for given category and maximum receivers
function GENERATE_RECEIVER_LIST ($cat, $receiver, $mode="") {
	global $_CONFIG;

	// Init variables
	$CAT_TABS     = "%s";
	$CAT_WHERE    = "";
	$receiverList = "";

	// Secure data
	$cat      = bigintval($cat);
	$receiver = bigintval($receiver);

	// Is the receiver zero and mode set?
	if (($receiver == 0) && (!empty($mode))) {
		// Auto-fix receiver maximum
		$receiver = GET_TOTAL_RECEIVERS($mode);
	} // END - if

	// Category given?
	if ($cat > 0) {
		// Select category
		$CAT_TABS  = "LEFT JOIN "._MYSQL_PREFIX."_user_cats AS c ON d.userid=c.userid";
		$CAT_WHERE = " AND c.cat_id=%s";
	} // END - if

	// Exclude users in holiday?
	if (GET_EXT_VERSION("holiday") >= "0.1.3") {
		// Add something for the holiday extension
		$CAT_WHERE .= " AND d.holiday_active='N'";
	} // END - if

	if ((EXT_IS_ACTIVE("html_mail")) && ($mode == "html")) {
		// Only include HTML receivers
		$result = SQL_QUERY_ESC("SELECT d.userid FROM `"._MYSQL_PREFIX."_user_data` AS d ".$CAT_TABS." WHERE d.status='CONFIRMED' AND d.html='Y'".$CAT_WHERE." ORDER BY d.%s %s LIMIT %s",
		 array($cat, getConfig('order_select'), getConfig('order_mode'), $receiver), __FILE__, __LINE__);
	} else {
		// Include all
		$result = SQL_QUERY_ESC("SELECT d.userid FROM `"._MYSQL_PREFIX."_user_data` AS d ".$CAT_TABS." WHERE d.status='CONFIRMED'".$CAT_WHERE." ORDER BY d.%s %s LIMIT %s",
		 array($cat, getConfig('order_select'), getConfig('order_mode'), $receiver), __FILE__, __LINE__);
	}

	// Entries found?
	if ((SQL_NUMROWS($result) >= $receiver) && ($receiver > 0)) {
		// Load all entries
		while (list($REC) = SQL_FETCHROW($result)) {
			// Add receiver when not empty
			if (!empty($REC)) $receiverList .= $REC.";";
		} // END - while

		// Free memory
		SQL_FREERESULT($result);

		// Remove trailing semicolon
		$receiverList = substr($receiverList, 0, -1);
	} // END - if

	// Return list
	return $receiverList;
}

// Get timestamp for given stats type and data
function USER_STATS_GET_TIMESTAMP ($type, $data, $uid = 0) {
	// Default timestamp is zero
	$stamp = 0;

	// User id set?
	if ((isset($GLOBALS['userid'])) && ($uid == 0)) {
		$uid = $GLOBALS['userid'];
	} // END - if

	// Is the extension installed and updated?
	if ((!EXT_IS_ACTIVE("sql_patches")) || (EXT_VERSION_IS_OLDER("sql_patches", "0.5.6"))) {
		// Return zero here
		return $stamp;
	} // END - if

	// Try to find the entry
	$result = SQL_QUERY_ESC("SELECT UNIX_TIMESTAMP(`inserted`) AS `stamp`
FROM "._MYSQL_PREFIX."_user_stats_data
WHERE userid=%s AND stats_type='%s' AND stats_data='%s'
LIMIT 1",
		array(bigintval($uid), $type, $data), __FILE__, __LINE__);

	// Is the entry there?
	if (SQL_NUMROWS($result) == 1) {
		// Get this stamp
		list($stamp) = SQL_FETCHROW($result);
	} // END - if

	// Free result
	SQL_FREERESULT($result);

	// Return stamp
	return $stamp;
}

// Inserts user stats
function USER_STATS_INSERT_RECORD ($uid, $type, $data) {
	// Is the extension installed and updated?
	if ((!EXT_IS_ACTIVE("sql_patches")) || (EXT_VERSION_IS_OLDER("sql_patches", "0.5.6"))) {
		// Return zero here
		return false;
	} // END - if

	// Does it exist?
	if ((!USER_STATS_GET_TIMESTAMP($type, $data, $uid)) && (!is_array($data))) {
		// Then insert it!
		SQL_QUERY_ESC("INSERT INTO "._MYSQL_PREFIX."_user_stats_data (`userid`,`stats_type`,`stats_data`) VALUES (%s,'%s','%s')",
			array(bigintval($uid), $type, $data), __FILE__, __LINE__);
	} elseif (is_array($data)) {
		// Invalid data!
		DEBUG_LOG(__FUNCTION__, __LINE__, " uid={$uid},type={$type},data={".gettype($data).": Invalid statistics data type!");
	}
}

// "Getter" for array for user refs and points in given level
function GET_USER_REF_POINTS ($uid, $level) {
	global $_CONFIG;

	//* DEBUG: */ print "----------------------- <font color=\"#00aa00\">".__FUNCTION__." - ENTRY</font> ------------------------<ul><li>\n";
	// Default is no refs and no nickname
	$ADD = "";
	$refs = array();

	// Do we have nickname extension installed?
	if (EXT_IS_ACTIVE("nickname")) {
		$ADD = ", ud.nickname";
	} // END - if

	// Get refs from database
	$result = SQL_QUERY_ESC("SELECT ur.id, ur.refid, ud.status, ud.last_online, ud.mails_confirmed, ud.emails_received".$ADD."
FROM "._MYSQL_PREFIX."_user_refs AS ur
LEFT JOIN "._MYSQL_PREFIX."_user_points AS up
ON ur.refid=up.userid AND ur.level=0
LEFT JOIN `"._MYSQL_PREFIX."_user_data` AS ud
ON ur.refid=ud.userid
WHERE ur.userid=%s AND ur.level=%s
ORDER BY ur.refid ASC",
		array(bigintval($uid), bigintval($level)), __FILE__, __LINE__);

	// Are there some entries?
	if (SQL_NUMROWS($result) > 0) {
		// Fetch all entries
		while ($row = SQL_FETCHARRAY($result)) {
			// Get total points of this user
			$row['points'] = GET_TOTAL_DATA($row['refid'], "user_points", "points") - GET_TOTAL_DATA($row['refid'], "user_data", "used_points");

			// Get unconfirmed mails
			$row['unconfirmed']  = GET_TOTAL_DATA($row['refid'], "user_links", "id", "userid", true);

			// Init clickrate with zero
			$row['clickrate'] = 0;

			// Is at least one mail received?
			if ($row['emails_received'] > 0) {
				// Calculate clickrate
				$row['clickrate'] = ($row['mails_confirmed'] / $row['emails_received'] * 100);
			} // END - if

			// Activity is "active" by default because if autopurge is not installed
			$row['activity'] = MEMBER_ACTIVITY_ACTIVE;

			// Is autopurge installed and the user inactive?
			if ((EXT_IS_ACTIVE("autopurge")) && ((time() - getConfig('ap_inactive_since')) >= $row['last_online']))  {
				// Inactive user!
				$row['activity'] = MEMBER_ACTIVITY_INACTIVE;
			} // END - if

			// Remove some entries
			unset($row['mails_confirmed']);
			unset($row['emails_received']);
			unset($row['last_online']);

			// Add row
			$refs[$row['id']] = $row;
		} // END - while
	} // END - if

	// Free result
	SQL_FREERESULT($result);

	// Return result
	//* DEBUG: */ print "</li></ul>----------------------- <font color=\"#aa0000\">".__FUNCTION__." - EXIT</font> ------------------------<br />\n";
	return $refs;
}

// Recuced the amount of received emails for the receipients for given email
function REDUCT_RECIPIENT_RECEIVED_MAILS ($column, $id, $count) {
	// Search for mail in database
	$result = SQL_QUERY_ESC("SELECT `userid` FROM `"._MYSQL_PREFIX."_user_links` WHERE `%s`=%s ORDER BY `userid` ASC LIMIT %s",
		array($column, bigintval($id), $count), __FILE__, __LINE__);

	// Are there entries?
	if (SQL_NUMROWS($result) > 0) {
		// Now load all userids for one big query!
		$UIDs = array();
		while (list($uid) = SQL_FETCHROW($result)) {
			$UIDs[$uid] = $uid;
		} // END - while

		// Now update all user accounts
		SQL_QUERY_ESC("UPDATE `"._MYSQL_PREFIX."_user_data` SET `emails_received`=`emails_received`-1 WHERE `userid` IN (%s) LIMIT %s",
			array(implode(",", $UIDs), count($UIDs)), __FILE__, __LINE__);
	} // END - if

	// Free result
	SQL_FREERESULT($result);
}

// [EOF]
?>
